from injectark import StrategyBuilder
from .altair import altair
from .base import base
from .check import check
from .json import json
from .rst import rst



strategy_builder = StrategyBuilder({
    'base': base,
    'altair': altair,
    'check': check,
    'json': json,
    'rst': rst
})



from ..core.common import Config
from ..application.domain.common import QueryParser
from ..core.data.json import (
    init_json_database, JsonRepository,
    JsonClassificationRepository, JsonClassifierRepository,
    JsonLinkRepository, JsonScheduleRepository,
    JsonSlotRepository, JsonTaskRepository)
from .altair_factory import AltairFactory


class JsonFactory(AltairFactory):
    def __init__(self, config: Config) -> None:
        super().__init__(config)
        self.config = config
        self.param_dir = self.config['param_dir']
        self.result_dir = self.config['result_dir']

    def json_classification_repository(self, query_parser: QueryParser,
                                       ) -> JsonClassificationRepository:
        repository = JsonClassificationRepository(
            self.param_dir, query_parser, file_suffix='param')
        return repository

    def json_classifier_repository(self, query_parser: QueryParser,
                                   ) -> JsonClassifierRepository:
        repository = JsonClassifierRepository(
            self.param_dir, query_parser, file_suffix='param')
        return repository

    def json_link_repository(self, query_parser: QueryParser,
                             ) -> JsonLinkRepository:
        repository = JsonLinkRepository(
            self.param_dir, query_parser, file_suffix='param')
        return repository

    def json_schedule_repository(self, query_parser: QueryParser,
                                 ) -> JsonScheduleRepository:
        repository = JsonScheduleRepository(
            self.result_dir, query_parser, file_suffix='result')
        return repository

    def json_slot_repository(self, query_parser: QueryParser,
                             ) -> JsonSlotRepository:
        repository = JsonSlotRepository(
            self.result_dir, query_parser, file_suffix='result')
        return repository

    def json_task_repository(self, query_parser: QueryParser,
                             ) -> JsonTaskRepository:
        repository = JsonTaskRepository(
            self.param_dir, query_parser, file_suffix='param')
        return repository



rst = {
    "RstAnalyzer": {
        "method": "rst_analyzer"
    },
    "RstLoader": {
        "method": "rst_loader"
    },
    "TaskRepository": {
        "method": "rst_task_repository"
    },
    "LinkRepository": {
        "method": "rst_link_repository"
    },
    "ClassificationRepository": {
        "method": "rst_classification_repository"
    }
}



base = {
    "QueryParser": {
        "method": "query_parser"
    },
    "TaskRepository": {
        "method": "memory_task_repository"
    },
    "LinkRepository": {
        "method": "memory_link_repository"
    },
    "ClassifierRepository": {
        "method": "memory_classifier_repository"
    },
    "ClassificationRepository": {
        "method": "memory_classification_repository"
    },
    "SlotRepository": {
        "method": "memory_slot_repository"
    },
    "PlotService": {
        "method": "memory_plot_service"
    },
    "ScheduleRepository": {
        "method": "memory_schedule_repository"
    },
    "EstimationManager": {
        "method": "estimation_manager"
    },
    "EstimarkInformer": {
        "method": "standard_estimark_informer"
    }
}



from injectark import Factory
from ..core.common import Config
from ..application.domain.common import QueryParser
from ..application.domain.repositories import (
    TaskRepository, MemoryTaskRepository,
    LinkRepository, MemoryLinkRepository,
    ClassifierRepository, MemoryClassifierRepository,
    ClassificationRepository, MemoryClassificationRepository,
    ScheduleRepository, MemoryScheduleRepository,
    SlotRepository, MemorySlotRepository)
from ..application.domain.services import PlotService, MemoryPlotService
from ..application.managers import EstimationManager
from ..application.informers import StandardEstimarkInformer


class BaseFactory(Factory):
    def __init__(self, config: Config) -> None:
        super().__init__(config)
        self.config = config

    def query_parser(self) -> QueryParser:
        return QueryParser()

    def memory_task_repository(
            self, query_parser: QueryParser
    ) -> MemoryTaskRepository:
        return MemoryTaskRepository(query_parser)

    def memory_link_repository(
            self, query_parser: QueryParser
    ) -> MemoryLinkRepository:
        return MemoryLinkRepository(query_parser)

    def memory_classifier_repository(
            self, query_parser: QueryParser
    ) -> MemoryClassifierRepository:
        return MemoryClassifierRepository(query_parser)

    def memory_classification_repository(
            self, query_parser: QueryParser
    ) -> MemoryClassificationRepository:
        return MemoryClassificationRepository(query_parser)

    def memory_schedule_repository(
            self, query_parser: QueryParser
    ) -> MemoryScheduleRepository:
        return MemoryScheduleRepository(query_parser)

    def memory_slot_repository(
            self, query_parser: QueryParser
    ) -> MemorySlotRepository:
        return MemorySlotRepository(query_parser)

    def memory_plot_service(self) -> MemoryPlotService:
        return MemoryPlotService()

    def estimation_manager(
            self, task_repository: TaskRepository,
            classifier_repository: ClassifierRepository,
            classification_repository: ClassificationRepository,
            link_repository: LinkRepository,
            schedule_repository: ScheduleRepository,
            slot_repository: SlotRepository,
            plot_service: PlotService
    ) -> EstimationManager:
        return EstimationManager(
            task_repository, classifier_repository,
            classification_repository, link_repository,
            schedule_repository, slot_repository, plot_service)

    def standard_estimark_informer(
            self, task_repository: TaskRepository,
            link_repository: LinkRepository,
            classifier_repository: ClassifierRepository,
            schedule_repository: ScheduleRepository,
            slot_repository: SlotRepository
    ) -> StandardEstimarkInformer:
        return StandardEstimarkInformer(
            task_repository, link_repository, classifier_repository,
            schedule_repository, slot_repository)



json = {
    "ClassificationRepository": {
        "method": "json_classification_repository"
    },
    "ClassifierRepository": {
        "method": "json_classifier_repository"
    },
    "LinkRepository": {
        "method": "json_link_repository"
    },
    "ScheduleRepository": {
        "method": "json_schedule_repository"
    },
    "SlotRepository": {
        "method": "json_slot_repository"
    },
    "TaskRepository": {
        "method": "json_task_repository"
    }
}
